%% GENERAL INFORMATION
% Generate a data set for the discrete-time simulator module.
%
% Author: Piotr Jaglarz (pjaglarz@student.agh.edu.pl)
% Date: 04.2014

%% INPUT DATA FOR THE SIMULATOR MODULE

function [ id, timeline, timeline_add, filename ] = gen_input_data( N, distribution, i_los )


% Load the parameters generated by the optimization module
	load( '../modul_optymalizacyjny/input' );

% Number of failure-repair pairs (down/up events) for each link

	% N = 1600;

% Events (arcs) - array of failure and repair model:
%	Events(n).failure_model - distribution of TTF (Time-to-Failure) times for the link:
%		0: exponential distribution (values in range: [param_1, param_2]; param_1 < param_2)
%		1: normal distribution (param_1 = mean, param_2 = std dev)
%	Events(n).failure_model_param_1 - the first parameter for the failure_model
%	Events(n).failure_model_param_2 - the second parameter for the failure_model
%	Events(n).repair_model - distribution of TTR (Time-to-Repair) times for the link:
%		0: uniform distribution (values in range: [param_1, param_2]; param_1 < param_2)
%		1: normal distribution (param_1 = mean, param_2 = std dev)
%	Events(n).repair_model_param_1 - the first parameter for the repair_model
%	Events(n).repair_model_param_2 - the second parameter for the repair_model

    Events = struct( ...
			'failure_model', {}, ...
			'failure_model_param_1', {}, ...
			'failure_model_param_2', {}, ...
			'repair_model', {}, ...
			'repair_model_param_1', {}, ...
			'repair_model_param_2', {} ...
	);

% Line Failures
%  Line failures can, for instance, be caused by a failure in the fiberoptic cable or the
%  failure of an OA or WDM line system. An assumption often made is that fiber
%  failures within a single fiberoptic cable are completely dependent, because most
%  failures are caused by dig ups, affecting all fibers within the cable. For physical
%  cables the MTBF can be specified using the cable cut (CC) metric. This is the
%  average cable length that results in a single cable cut per year (e.g., CC ? 450 km
%  means that per 450km cable, there will be on average one cable cut each year). This
%  expresses the fact that the probability to have a cable cut is larger for a longer link.
%
% The MTBF of the cable is then calculated as
%   MTBF(hours) = (CC * 365 * 24) / Length of the cable

	for i = 1 : length( A )
        if strcmp( distribution, 'exp' )
            Events( i ).failure_model = 0;
            Events( i ).failure_model_param_1 = ( 0.5 * 450 * 365 * 24 ) / A( i, 5 );
            Events( i ).repair_model = 0;
            Events( i ).repair_model_param_1 = 12;
        elseif strcmp( distribution, 'pareto' )
            Events( i ).failure_model = 1;
            Events( i ).failure_model_param_1 = 1 / ( 3.18443e-07 * ( 1 + 9 * A( i, 5 ) / max( A( :, 5 ) ) ) );
            Events( i ).repair_model = 1;
            Events( i ).repair_model_param_1 = 60 * ( 1 + 9 * A( i, 5 ) / max( A( :, 5 ) ) );
            Events( i ).repair_model_param_2 = 2.3;
        end
	end
    
% For each link, generate a vector of time intervals (expressed as the 
% number of time units) between consecutive status changes (down/up events).
% Please note that TTF and TTR times usually have different distributions.

	events = zeros( length( Events ), 2*N );
	failures = zeros( length( Events ), N );
	repairs = zeros( length( Events ), N );
	
    %rng('shuffle');     % Generate Random Numbers That Are Different
	rng(i_los);
    for i = 1 : length( Events )
		
		% Failure model
		
		if Events( i ).failure_model == 0
			% Exponential distribution
			failures( i, : ) = exprnd( Events( i ).failure_model_param_1, 1, N );
            
		elseif Events( i ).failure_model == 1
			% % Pareto distribution
			% %failures( i, : ) = gprnd( 1 / Events( i ).failure_model_param_2, Events( i ).failure_model_param_1 / Events( i ).failure_model_param_2, Events( i ).failure_model_param_1, 1, N );
            % failures( i, : ) = prnd( Events( i ).failure_model_param_1, Events( i ).failure_model_param_2, 1, N ) / 60;
            
            % Exponential distribution
			failures( i, : ) = exprnd( Events( i ).failure_model_param_1, 1, N ) / 60;
            
		end
		
		% Repair model
		
		if Events( i ).repair_model == 0
			% Exponential distribution
			repairs( i, : ) = exprnd( Events( i ).repair_model_param_1, 1, N );
            
		elseif Events( i ).repair_model == 1
			% Pareto distribution
			%repairs( i, : ) = gprnd( 1 / Events( i ).repair_model_param_2, Events( i ).repair_model_param_1 / Events( i ).repair_model_param_2, Events( i ).repair_model_param_1, 1, N ) / 60;
            repairs( i, : ) = prnd( Events( i ).repair_model_param_1, Events( i ).repair_model_param_2, 1, N ) / 60;
            
		end
		
	end
	
	% Join the time intervals to form failure-repair pairs
	
	z = 1;
	for j = 1 : N
		% Copy a column from the 'failures' matrix
		events( :, z ) = failures( :, j );
		z = z + 1;
		
		% Copy the corresponding column from the 'repairs' matrix
		events( :, z ) = repairs( :, j );
		z = z + 1;
	end
	
% Generate the timeline for the simulation, and, at the same time, convert 
% each time interval between consecutive events into the number of time 
% units counted from the reference time (start of the simulation)
	
	timelines = zeros( size( events, 1 ) * size( events, 2 ), 2 );
    timelines( 1, : ) = 0;
    z = 2;
	
	for i = 1 : size( events, 1 )
		time_counter = 0;
		
		for j = 1 : size( events, 2 )
			time_counter = time_counter + events( i, j );
			events( i, j ) = time_counter;
			timelines( z, 1 ) = time_counter;
            timelines( z, 2 ) = i;
			z = z + 1;
		end
	end
	
	% Strip duplicate entries, sort in ascending order
    timelines = sortrows( timelines );
    timeline = zeros( 1, length( unique( timelines) ) - length( A ) );
    timeline_add = zeros( length( unique( timelines) ) - length( A ), 2 );
    timeline( 1 ) = 0;
    timeline_add( 1 ) = 0;

    y = 1;
    for i = 2 : length( timelines )
        if timelines( i - 1, 1 ) ~= timelines( i, 1 )
            y = y + 1;
            timeline( y ) = timelines( i, 1 );
            z = 1;
            timeline_add( y, z ) = timelines( i, 2 );
        else
            z = z + 1;
            timeline_add( y, z ) = timelines( i, 2 );
        end
    end

    % Generate failures data id
    id = round( 10000 * rand );

%% SAVE THE SELECTED VARIABLES INTO A .mat FILE

%     save( 'events', 'id', 'N', 'events', 'timeline', 'timeline_add', 'filename', 'failures', 'repairs' );
